"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var test_helper_1 = require("../helpers/test-helper");
var marble_testing_1 = require("../helpers/marble-testing");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
describe('static merge(...observables)', function () {
    it('should merge cold and cold', function () {
        var e1 = (0, marble_testing_1.cold)('---a-----b-----c----|');
        var e1subs = '^                   !';
        var e2 = (0, marble_testing_1.cold)('------x-----y-----z----|');
        var e2subs = '^                      !';
        var expected = '---a--x--b--y--c--z----|';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should return itself when try to merge single observable', function () {
        var e1 = (0, rxjs_1.of)('a');
        var result = (0, rxjs_1.merge)(e1);
        (0, chai_1.expect)(e1).to.equal(result);
    });
    it('should merge hot and hot', function () {
        var e1 = (0, marble_testing_1.hot)('---a---^-b-----c----|');
        var e1subs = '^            !';
        var e2 = (0, marble_testing_1.hot)('-----x-^----y-----z----|');
        var e2subs = '^               !';
        var expected = '--b--y--c--z----|';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge hot and cold', function () {
        var e1 = (0, marble_testing_1.hot)('---a-^---b-----c----|');
        var e1subs = '^              !';
        var e2 = (0, marble_testing_1.cold)('--x-----y-----z----|');
        var e2subs = '^                  !';
        var expected = '--x-b---y-c---z----|';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge parallel emissions', function () {
        var e1 = (0, marble_testing_1.hot)('---a----b----c----|');
        var e1subs = '^                 !';
        var e2 = (0, marble_testing_1.hot)('---x----y----z----|');
        var e2subs = '^                 !';
        var expected = '---(ax)-(by)-(cz)-|';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge empty and empty', function () {
        var e1 = (0, marble_testing_1.cold)('|');
        var e1subs = '(^!)';
        var e2 = (0, marble_testing_1.cold)('|');
        var e2subs = '(^!)';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe('|');
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge three empties', function () {
        var e1 = (0, marble_testing_1.cold)('|');
        var e1subs = '(^!)';
        var e2 = (0, marble_testing_1.cold)('|');
        var e2subs = '(^!)';
        var e3 = (0, marble_testing_1.cold)('|');
        var e3subs = '(^!)';
        var result = (0, rxjs_1.merge)(e1, e2, e3);
        (0, marble_testing_1.expectObservable)(result).toBe('|');
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
        (0, marble_testing_1.expectSubscriptions)(e3.subscriptions).toBe(e3subs);
    });
    it('should merge never and empty', function () {
        var e1 = (0, marble_testing_1.cold)('-');
        var e1subs = '^';
        var e2 = (0, marble_testing_1.cold)('|');
        var e2subs = '(^!)';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe('-');
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge never and never', function () {
        var e1 = (0, marble_testing_1.cold)('-');
        var e1subs = '^';
        var e2 = (0, marble_testing_1.cold)('-');
        var e2subs = '^';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe('-');
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge empty and throw', function () {
        var e1 = (0, marble_testing_1.cold)('|');
        var e1subs = '(^!)';
        var e2 = (0, marble_testing_1.cold)('#');
        var e2subs = '(^!)';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe('#');
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge hot and throw', function () {
        var e1 = (0, marble_testing_1.hot)('--a--b--c--|');
        var e1subs = '(^!)';
        var e2 = (0, marble_testing_1.cold)('#');
        var e2subs = '(^!)';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe('#');
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge never and throw', function () {
        var e1 = (0, marble_testing_1.cold)('-');
        var e1subs = '(^!)';
        var e2 = (0, marble_testing_1.cold)('#');
        var e2subs = '(^!)';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe('#');
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge empty and eventual error', function () {
        var e1 = (0, marble_testing_1.cold)('|');
        var e1subs = '(^!)';
        var e2 = (0, marble_testing_1.hot)('-------#');
        var e2subs = '^------!';
        var expected = '-------#';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge hot and error', function () {
        var e1 = (0, marble_testing_1.hot)('--a--b--c--|');
        var e1subs = '^      !    ';
        var e2 = (0, marble_testing_1.hot)('-------#    ');
        var e2subs = '^      !    ';
        var expected = '--a--b-#    ';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge never and error', function () {
        var e1 = (0, marble_testing_1.hot)('-');
        var e1subs = '^      !';
        var e2 = (0, marble_testing_1.hot)('-------#');
        var e2subs = '^      !';
        var expected = '-------#';
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, marble_testing_1.expectObservable)(result).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
        (0, marble_testing_1.expectSubscriptions)(e2.subscriptions).toBe(e2subs);
    });
    it('should merge single lowerCaseO into RxJS Observable', function () {
        var e1 = (0, test_helper_1.lowerCaseO)('a', 'b', 'c');
        var result = (0, rxjs_1.merge)(e1);
        (0, chai_1.expect)(result).to.be.instanceof(rxjs_1.Observable);
        (0, marble_testing_1.expectObservable)(result).toBe('(abc|)');
    });
    it('should merge two lowerCaseO into RxJS Observable', function () {
        var e1 = (0, test_helper_1.lowerCaseO)('a', 'b', 'c');
        var e2 = (0, test_helper_1.lowerCaseO)('d', 'e', 'f');
        var result = (0, rxjs_1.merge)(e1, e2);
        (0, chai_1.expect)(result).to.be.instanceof(rxjs_1.Observable);
        (0, marble_testing_1.expectObservable)(result).toBe('(abcdef|)');
    });
});
describe('merge(...observables, Scheduler)', function () {
    it('should merge single lowerCaseO into RxJS Observable', function () {
        var e1 = (0, test_helper_1.lowerCaseO)('a', 'b', 'c');
        var result = (0, rxjs_1.merge)(e1, rxTestScheduler);
        (0, chai_1.expect)(result).to.be.instanceof(rxjs_1.Observable);
        (0, marble_testing_1.expectObservable)(result).toBe('(abc|)');
    });
});
describe('merge(...observables, Scheduler, number)', function () {
    it('should handle concurrency limits', function () {
        var e1 = (0, marble_testing_1.cold)('---a---b---c---|');
        var e2 = (0, marble_testing_1.cold)('-d---e---f--|');
        var e3 = (0, marble_testing_1.cold)('---x---y---z---|');
        var expected = '-d-a-e-b-f-c---x---y---z---|';
        (0, marble_testing_1.expectObservable)((0, rxjs_1.merge)(e1, e2, e3, 2)).toBe(expected);
    });
    it('should handle scheduler', function () {
        var e1 = (0, rxjs_1.of)('a');
        var e2 = (0, rxjs_1.of)('b').pipe((0, operators_1.delay)(20, rxTestScheduler));
        var expected = 'a-(b|)';
        (0, marble_testing_1.expectObservable)((0, rxjs_1.merge)(e1, e2, rxTestScheduler)).toBe(expected);
    });
    it('should handle scheduler with concurrency limits', function () {
        var e1 = (0, marble_testing_1.cold)('---a---b---c---|');
        var e2 = (0, marble_testing_1.cold)('-d---e---f--|');
        var e3 = (0, marble_testing_1.cold)('---x---y---z---|');
        var expected = '-d-a-e-b-f-c---x---y---z---|';
        (0, marble_testing_1.expectObservable)((0, rxjs_1.merge)(e1, e2, e3, 2, rxTestScheduler)).toBe(expected);
    });
    it('should use the scheduler even when one Observable is merged', function (done) {
        var e1Subscribed = false;
        var e1 = (0, rxjs_1.defer)(function () {
            e1Subscribed = true;
            return (0, rxjs_1.of)('a');
        });
        (0, rxjs_1.merge)(e1, rxjs_1.asyncScheduler)
            .subscribe({
            error: done,
            complete: function () {
                (0, chai_1.expect)(e1Subscribed).to.be.true;
                done();
            }
        });
        (0, chai_1.expect)(e1Subscribed).to.be.false;
    });
    it('should deem a single array argument to be an ObservableInput', function () {
        var array = ['foo', 'bar'];
        var expected = '(fb|)';
        (0, marble_testing_1.expectObservable)((0, rxjs_1.merge)(array)).toBe(expected, { f: 'foo', b: 'bar' });
    });
});
//# sourceMappingURL=merge-spec.js.map