"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var marble_testing_1 = require("../helpers/marble-testing");
var operators_1 = require("rxjs/operators");
var chai_1 = require("chai");
describe('onErrorResumeNext', function () {
    it('should continue with observables', function () {
        var s1 = (0, marble_testing_1.hot)('--a--b--#');
        var s2 = (0, marble_testing_1.cold)('--c--d--#');
        var s3 = (0, marble_testing_1.cold)('--e--#');
        var s4 = (0, marble_testing_1.cold)('--f--g--|');
        var subs1 = '^       !';
        var subs2 = '        ^       !';
        var subs3 = '                ^    !';
        var subs4 = '                     ^       !';
        var expected = '--a--b----c--d----e----f--g--|';
        (0, marble_testing_1.expectObservable)((0, rxjs_1.onErrorResumeNext)(s1, s2, s3, s4)).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(s1.subscriptions).toBe(subs1);
        (0, marble_testing_1.expectSubscriptions)(s2.subscriptions).toBe(subs2);
        (0, marble_testing_1.expectSubscriptions)(s3.subscriptions).toBe(subs3);
        (0, marble_testing_1.expectSubscriptions)(s4.subscriptions).toBe(subs4);
    });
    it('should continue array of observables', function () {
        var s1 = (0, marble_testing_1.hot)('--a--b--#');
        var s2 = (0, marble_testing_1.cold)('--c--d--#');
        var s3 = (0, marble_testing_1.cold)('--e--#');
        var s4 = (0, marble_testing_1.cold)('--f--g--|');
        var subs1 = '^       !';
        var subs2 = '        ^       !';
        var subs3 = '                ^    !';
        var subs4 = '                     ^       !';
        var expected = '--a--b----c--d----e----f--g--|';
        (0, marble_testing_1.expectObservable)((0, rxjs_1.onErrorResumeNext)([s1, s2, s3, s4])).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(s1.subscriptions).toBe(subs1);
        (0, marble_testing_1.expectSubscriptions)(s2.subscriptions).toBe(subs2);
        (0, marble_testing_1.expectSubscriptions)(s3.subscriptions).toBe(subs3);
        (0, marble_testing_1.expectSubscriptions)(s4.subscriptions).toBe(subs4);
    });
    it('should complete single observable throws', function () {
        var source = (0, marble_testing_1.hot)('#');
        var subs = '(^!)';
        var expected = '|';
        (0, marble_testing_1.expectObservable)((0, rxjs_1.onErrorResumeNext)(source)).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(source.subscriptions).toBe(subs);
    });
    it('should skip invalid sources and move on', function () {
        var results = [];
        (0, rxjs_1.onErrorResumeNext)((0, rxjs_1.of)(1), [2, 3, 4], { notValid: 'LOL' }, (0, rxjs_1.of)(5, 6))
            .subscribe({
            next: function (value) { return results.push(value); },
            complete: function () { return results.push('complete'); }
        });
        (0, chai_1.expect)(results).to.deep.equal([1, 2, 3, 4, 5, 6, 'complete']);
    });
    it('should call finalize after each sync observable', function () {
        var results = [];
        (0, rxjs_1.onErrorResumeNext)((0, rxjs_1.of)(1).pipe((0, operators_1.finalize)(function () { return results.push('finalize 1'); })), (0, rxjs_1.of)(2).pipe((0, operators_1.finalize)(function () { return results.push('finalize 2'); })), (0, rxjs_1.of)(3).pipe((0, operators_1.finalize)(function () { return results.push('finalize 3'); })), (0, rxjs_1.of)(4).pipe((0, operators_1.finalize)(function () { return results.push('finalize 4'); }))).subscribe({
            next: function (value) { return results.push(value); },
            complete: function () { return results.push('complete'); }
        });
        (0, chai_1.expect)(results).to.deep.equal([
            1, 'finalize 1',
            2, 'finalize 2',
            3, 'finalize 3',
            4, 'finalize 4',
            'complete'
        ]);
    });
});
//# sourceMappingURL=onErrorResumeNext-spec.js.map