"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var marble_testing_1 = require("../helpers/marble-testing");
var operators_1 = require("rxjs/operators");
var rxjs_1 = require("rxjs");
describe('repeat operator', function () {
    it('should resubscribe count number of times', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--|                ');
        var subs = ['^       !                ',
            '        ^       !        ',
            '                ^       !'];
        var expected = '--a--b----a--b----a--b--|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(3))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should resubscribe multiple times', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--|                        ');
        var subs = ['^       !                        ',
            '        ^       !                ',
            '                ^       !        ',
            '                        ^       !'];
        var expected = '--a--b----a--b----a--b----a--b--|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(2), (0, operators_1.repeat)(2))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should complete without emit when count is zero', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--|');
        var subs = [];
        var expected = '|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(0))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should emit source once when count is one', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--|');
        var subs = '^       !';
        var expected = '--a--b--|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(1))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should repeat until gets unsubscribed', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--|      ');
        var subs = ['^       !      ',
            '        ^     !'];
        var unsub = '              !';
        var expected = '--a--b----a--b-';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(10)), unsub).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should be able to repeat indefinitely until unsubscribed', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--|                                    ');
        var subs = ['^       !                                    ',
            '        ^       !                            ',
            '                ^       !                    ',
            '                        ^       !            ',
            '                                ^       !    ',
            '                                        ^   !'];
        var unsub = '                                            !';
        var expected = '--a--b----a--b----a--b----a--b----a--b----a--';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)()), unsub).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should not break unsubscription chain when unsubscribed explicitly', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--|                                    ');
        var subs = ['^       !                                    ',
            '        ^       !                            ',
            '                ^       !                    ',
            '                        ^       !            ',
            '                                ^       !    ',
            '                                        ^   !'];
        var unsub = '                                            !';
        var expected = '--a--b----a--b----a--b----a--b----a--b----a--';
        var result = e1.pipe((0, operators_1.mergeMap)(function (x) { return (0, rxjs_1.of)(x); }), (0, operators_1.repeat)(), (0, operators_1.mergeMap)(function (x) { return (0, rxjs_1.of)(x); }));
        (0, marble_testing_1.expectObservable)(result, unsub).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should consider negative count as no repeat, and return EMPTY', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--|                                    ');
        var unsub = '                                            !';
        var expected = '|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(-1)), unsub).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe([]);
    });
    it('should always teardown before starting the next cycle', function () { return __awaiter(void 0, void 0, void 0, function () {
        var results, source;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    results = [];
                    source = new rxjs_1.Observable(function (subscriber) {
                        Promise.resolve().then(function () {
                            subscriber.next(1);
                            Promise.resolve().then(function () {
                                subscriber.next(2);
                                Promise.resolve().then(function () {
                                    subscriber.complete();
                                });
                            });
                        });
                        return function () {
                            results.push('teardown');
                        };
                    });
                    return [4, source.pipe((0, operators_1.repeat)(3)).forEach(function (value) { return results.push(value); })];
                case 1:
                    _a.sent();
                    (0, chai_1.expect)(results).to.deep.equal([1, 2, 'teardown', 1, 2, 'teardown', 1, 2, 'teardown']);
                    return [2];
            }
        });
    }); });
    it('should always teardown before starting the next cycle, even when synchronous', function () {
        var results = [];
        var source = new rxjs_1.Observable(function (subscriber) {
            subscriber.next(1);
            subscriber.next(2);
            subscriber.complete();
            return function () {
                results.push('teardown');
            };
        });
        var subscription = source.pipe((0, operators_1.repeat)(3)).subscribe({
            next: function (value) { return results.push(value); },
            complete: function () { return results.push('complete'); }
        });
        (0, chai_1.expect)(subscription.closed).to.be.true;
        (0, chai_1.expect)(results).to.deep.equal([1, 2, 'teardown', 1, 2, 'teardown', 1, 2, 'complete', 'teardown']);
    });
    it('should not complete when source never completes', function () {
        var e1 = (0, marble_testing_1.cold)('-');
        var e1subs = '^';
        var expected = '-';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(3))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
    });
    it('should not complete when source does not completes', function () {
        var e1 = (0, marble_testing_1.cold)('-');
        var unsub = '                              !';
        var subs = '^                             !';
        var expected = '-';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(3)), unsub).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should complete immediately when source does not complete without emit but count is zero', function () {
        var e1 = (0, marble_testing_1.cold)('-');
        var subs = [];
        var expected = '|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(0))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should complete immediately when source does not complete but count is zero', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--');
        var subs = [];
        var expected = '|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(0))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should emit source once and does not complete when source emits but does not complete', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--');
        var subs = ['^       '];
        var expected = '--a--b--';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(3))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should complete when source is empty', function () {
        var e1 = (0, marble_testing_1.cold)('|');
        var e1subs = ['(^!)', '(^!)', '(^!)'];
        var expected = '|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(3))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
    });
    it('should complete when source does not emit', function () {
        var e1 = (0, marble_testing_1.cold)('----|        ');
        var subs = ['^   !        ',
            '    ^   !    ',
            '        ^   !'];
        var expected = '------------|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(3))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should complete immediately when source does not emit but count is zero', function () {
        var e1 = (0, marble_testing_1.cold)('----|');
        var subs = [];
        var expected = '|';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(0))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should raise error when source raises error', function () {
        var e1 = (0, marble_testing_1.cold)('--a--b--#');
        var subs = '^       !';
        var expected = '--a--b--#';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(2))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(subs);
    });
    it('should raises error if source throws', function () {
        var e1 = (0, marble_testing_1.cold)('#');
        var e1subs = '(^!)';
        var expected = '#';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(3))).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
    });
    it('should raises error if source throws when repeating infinitely', function () {
        var e1 = (0, marble_testing_1.cold)('#');
        var e1subs = '(^!)';
        var expected = '#';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)())).toBe(expected);
        (0, marble_testing_1.expectSubscriptions)(e1.subscriptions).toBe(e1subs);
    });
    it('should raise error after first emit succeed', function () {
        var repeated = false;
        var e1 = (0, marble_testing_1.cold)('--a--|').pipe((0, operators_1.map)(function (x) {
            if (repeated) {
                throw 'error';
            }
            else {
                repeated = true;
                return x;
            }
        }));
        var expected = '--a----#';
        (0, marble_testing_1.expectObservable)(e1.pipe((0, operators_1.repeat)(2))).toBe(expected);
    });
    it('should repeat a synchronous source (multicasted and refCounted) multiple times', function (done) {
        var expected = [1, 2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3];
        (0, rxjs_1.of)(1, 2, 3).pipe((0, operators_1.multicast)(function () { return new rxjs_1.Subject(); }), (0, operators_1.refCount)(), (0, operators_1.repeat)(5)).subscribe(function (x) { (0, chai_1.expect)(x).to.equal(expected.shift()); }, function (x) {
            done(new Error('should not be called'));
        }, function () {
            (0, chai_1.expect)(expected.length).to.equal(0);
            done();
        });
    });
    it('should stop listening to a synchronous observable when unsubscribed', function () {
        var sideEffects = [];
        var synchronousObservable = new rxjs_1.Observable(function (subscriber) {
            for (var i = 0; !subscriber.closed && i < 10; i++) {
                sideEffects.push(i);
                subscriber.next(i);
            }
        });
        synchronousObservable.pipe((0, operators_1.repeat)(), (0, operators_1.take)(3)).subscribe(function () { });
        (0, chai_1.expect)(sideEffects).to.deep.equal([0, 1, 2]);
    });
});
//# sourceMappingURL=repeat-spec.js.map